"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
exports.enableDracoLoader = enableDracoLoader;
exports.enableKtx2Loader = enableKtx2Loader;
exports.legacyGLTFLoader = exports.glTFLoader = void 0;
var THREE = _interopRequireWildcard(require("three"));
var _GLTFLoader = require("../ThreeExtended/loaders/GLTFLoader");
var _DRACOLoader = require("../ThreeExtended/loaders/DRACOLoader");
var _KTX2Loader = require("../ThreeExtended/loaders/KTX2Loader");
var _LegacyGLTFLoader = _interopRequireDefault(require("./deprecated/LegacyGLTFLoader"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const matrixChangeUpVectorYtoZ = new THREE.Matrix4().makeRotationX(Math.PI / 2);
const glTFLoader = new _GLTFLoader.GLTFLoader();
exports.glTFLoader = glTFLoader;
const legacyGLTFLoader = new _LegacyGLTFLoader.default();

/**
 * @module GLTFParser
 * @description Parses [glTF](https://www.khronos.org/gltf/) 1.0 and 2.0 files.
 *
 * Under the hood, glTF 2.0 files are parsed with THREE.GltfLoader() and GLTF 1.0 are parsed with the previous THREE
 * GltfLoader (for 1.0 glTF) that has been kept and maintained in iTowns.
 */

/**
 * Enable loading gltf files with [Draco](https://google.github.io/draco/) geometry extension.
 *
 * @param {String} path path to draco library folder containing the JS and WASM decoder libraries. They can be found in
 * [itowns examples](https://github.com/iTowns/itowns/tree/master/examples/libs/draco).
 * @param {Object} [config] optional configuration for Draco decoder (see threejs'
 * [setDecoderConfig](https://threejs.org/docs/index.html?q=draco#examples/en/loaders/DRACOLoader.setDecoderConfig) that
 * is called under the hood with this configuration for details.
 */
exports.legacyGLTFLoader = legacyGLTFLoader;
function enableDracoLoader(path, config) {
  if (!path) {
    throw new Error('Path to draco folder is mandatory');
  }
  const dracoLoader = new _DRACOLoader.DRACOLoader();
  dracoLoader.setDecoderPath(path);
  if (config) {
    dracoLoader.setDecoderConfig(config);
  }
  glTFLoader.setDRACOLoader(dracoLoader);
}

/**
 * Enable loading gltf files with [KTX2](https://www.khronos.org/ktx/) texture extension.
 *
 * @param {String} path path to ktx2 library folder containing the JS and WASM decoder libraries. They can be found in
 * [itowns examples](https://github.com/iTowns/itowns/tree/master/examples/libs/basis).
 * @param {THREE.WebGLRenderer} renderer the threejs renderer
 */
function enableKtx2Loader(path, renderer) {
  if (!path || !renderer) {
    throw new Error('Path to ktx2 folder and renderer are mandatory');
  }
  const ktx2Loader = new _KTX2Loader.KTX2Loader();
  ktx2Loader.setTranscoderPath(path);
  ktx2Loader.detectSupport(renderer);
  glTFLoader.setKTX2Loader(ktx2Loader);
}
var _default = {
  /** Parses a gltf buffer to an object with threejs structures and applies a y-up to z-up conversion to align with
   * itowns convention. Essentially calls THREE.GltfLoader.parse() for glTF 2.0 files and the legacy threejs parser
   * for gtTF 1.0 files.
   * @param {ArrayBuffer} buffer - the glTF asset to parse, as an ArrayBuffer, JSON string or object.
   * @param {String} path - the base path from which to find subsequent glTF resources such as textures and .bin data files.
   * @return {Promise} - a promise that resolves with an object containing an Object that contains loaded parts:
   * .scene, .scenes, .cameras, .animations, and .asset.
   */
  parse(buffer, path) {
    return new Promise((resolve, reject) => {
      if (!buffer || !path) {
        reject(new Error('[GLTFParser]: Buffer and path are mandatory to parse a glTF.'));
        return;
      }

      // Apply y-up (gltf convention) to z-up (itowns convention) conversion
      const onload = gltf => {
        gltf.scene.applyMatrix4(matrixChangeUpVectorYtoZ);
        resolve(gltf);
      };
      const onError = e => {
        reject(new Error(`[GLTFParser]: Failed to parse gltf with error: ${e}`));
      };
      const headerView = new DataView(buffer, 0, 20);
      const version = headerView.getUint32(4, true);
      if (version === 1) {
        legacyGLTFLoader.parse(buffer, path, onload, onError);
      } else {
        glTFLoader.parse(buffer, path, onload, onError);
      }
    });
  }
};
exports.default = _default;